function [RF_frame,RF_frame_Q, x_axis, y_axis,image_N_BIT]=build_b_mode(RF_frame, RF_frame_Q,  HEADER,Number_of_frame2show,image_bits,interpolated_ampl_curve,flag,low_freq_cuttoff, up_freq_cuttoff)
% Function returns RF data of single preselected frame and N bit image data
% and spatial coordinates of the image

%% IN:
% RF_frame - RF data of selected frame (I component)
% RF_frame_Q - RF data Q component (after HIlberts transform)
% HEADER - RF data acquisition parameters: 
% HEADER{1,1}.tx_frequency - ultrasonic wave (transmission) frequency 
% HEADER{1,1}.Length_of_RF_row - number of samples in each RF row
% HEADER{1,1}.Number_of_RF_rows - number of RF scanning lines in a window
% HEADER{1,1}.Sampling_period_ns - sampling period in ns
% HEADER{1,1}.BIT_ADC - number of ADC bits
% HEADER{1,1}.beam_x - start point x coordinates for each ultrasound beam in cm 
% HEADER{1,1}.beam_y - start point y coordinates for each ultrasound beam in cm
% HEADER{1,1}.angle - angle in radians (the angle is given relative to  the perpendicular to the center of the probes surface)
% Number_of_frame2show - selected frame for the display and analysis
% image_bits - number of image bits (typically - 8)
% interpolated_ampl_curve - TGC (time-gain compensation) ampolification curve points 
% flag - for band-pass (BP) filter type selection 0 for infinite impulse response (Butterworth type), 1 - for finite impulse response (FIR filter designed using the window method), 2 - turn of the filters
% low_freq_cuttoff - lower cut-off frequecy for BP filter 
% up_freq_cuttoff - upper cut-off frequecy for BP filter

%% OUT:
% RF_frame - RF data of selected frame 
% RF_frame_Q - RF data Q component (after HIlberts transform)
% x_axis, y_axis - spatial coordinates of B mode frame
% image_N_BIT - B mode image data (N bit)


Sampling_period_s=HEADER{1,Number_of_frame2show}.Sampling_period_ns*1e-9;  %% Sampling_period, seconds
Sampling_frequency=1/Sampling_period_s;                                    %% Sampling frequency, Hz

%% ------------------------------------------------------------------------------------------
%% Filtering of RF frame
RF_frame=band_pass_filter(RF_frame, low_freq_cuttoff, up_freq_cuttoff, Sampling_frequency, flag);
RF_frame_Q=band_pass_filter(RF_frame_Q, low_freq_cuttoff, up_freq_cuttoff, Sampling_frequency, flag);

%% ------------------------------------------------------------------------------------------

%% TGC amplification
RF_frame_ampl=RF_frame.*repmat(interpolated_ampl_curve,1,size(RF_frame,2)); 
RF_frame_Q_ampl = RF_frame_Q.*repmat(interpolated_ampl_curve,1,size(RF_frame,2));

%% RF data to envelope (abs(hilbert)) 
for i=1:size(RF_frame,2)
[envelope(:,i), Q_comp]=envelope_detection(RF_frame_ampl(:,i));
end

%% ------------------------------------------------------------------------------------------
%% Nonlinear suppression 
%% Mapping to 8 BIT image 
%% ------------------------------------------------------------------------------------------
alpha_=0.47;                                                               %% Coefficient of nonlinearity
num_gray_levels=2^image_bits;                                      %% Number of gray levels to map RF data 0...num_gray_levels
image_N_BIT=envelope_2_image(alpha_, num_gray_levels, HEADER{1,Number_of_frame2show}.sample_size, envelope);

%% ------------------------------------------------------------------------------------------
%% Probe type sector vs rectangle %% --------------------------------------------------------
if (mean(HEADER{1,Number_of_frame2show}.angle)~=0) 
    
angle_vector_radians=HEADER{1,Number_of_frame2show}.angle;                 %% Direction (angle) vector in radians  
%% Selection of transducer geometry (sector or rectangle) 
beam_x=HEADER{1,Number_of_frame2show}.beam_x;                              %% Start x coordinates of each beam, cm
beam_y=HEADER{1,Number_of_frame2show}.beam_y;                              %% Start y coordinates of each beam, cm

for j=1:length(angle_vector_radians)    

time_axis=0:Sampling_period_s:(HEADER{1,Number_of_frame2show}.Length_of_RF_row-1)*Sampling_period_s;      %% scanning time axis, seconds (starting from 0)
depth_axis=(time_axis*0.5*HEADER{1,Number_of_frame2show}.Speed_of_sound)*100;             %% scanning depth axis, in cm (starting from 0) 

depth_vectors=[depth_axis; zeros(1,length(depth_axis))];
%% Rotates vector respective to direction (angle is given relative to the perpendicular to the center of the probes surface) 
[depth_vectors]=rotation_matrix(depth_vectors, -angle_vector_radians(j));  

%% Shift respective to start points -------------------------------------- %%
y_axis(j,:)=depth_vectors(1,:)+beam_y(j);                                  % Depth axis (correct absolute coordinates of RF window), cm
x_axis(j,:)=depth_vectors(2,:)+beam_x(j);                                  % Width axis (correct absolute coordinates of RF window), cm

end

else %% (to verify rectangular image)
   
beam_x=HEADER{1,Number_of_frame2show}.beam_x;                              %% Start x coordinates of each beam, cm
beam_y=HEADER{1,Number_of_frame2show}.beam_y;                              %% Start y coordinates of each beam, cm

x_axis=beam_x;                                                             %% x coordinates_of B mode (correct absolute coordinates of sector width), in cm

depth_step=(Sampling_period_s*0.5*HEADER{1,Number_of_frame2show}.Speed_of_sound)*100;                     %% Scanning depth step, in cm                             
y_axis=beam_y:depth_step:beam_y+(HEADER{1,Number_of_frame2show}.Length_of_RF_row-1)*depth_step; %% scanning depth axis, in cm                                   

end